package no.marintek.hla.case_one;

import java.net.*;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import static marintek.hla.AttributeType.i_CHAR;
import static marintek.hla.AttributeType.i_DOUBLE;
import static marintek.hla.AttributeType.i_INT;
import marintek.hla.app.HLAApp;
import marintek.hla.app.HLAException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Scanner;
import java.io.IOException;
import de.beckhoff.jni.Convert;
import de.beckhoff.jni.JNIByteBuffer;
import de.beckhoff.jni.tcads.AmsAddr;
import de.beckhoff.jni.tcads.AdsCallDllFunction;

public class FakeSimo extends HLAApp {

	private static final double DT = 0.02;
	private static final double RAMP_STEPS = 1000;
		
	// class handles
	private int ch_body;
	private int ch_force;

	// attribute handles - MOP.Body
	private int hPos;
	private int hVel;
	private int hAcc;
	private int hPosEst;
	private int hVelEst;

	// attribute handles - MOP.ExternalForce
	private int ah_iInfoFor;
	private int ah_dInfoFor;
	private int ah_cInfoFor;
	private int ah_force;
	private int ah_attackPoint;

	// object handles
	private int[] oh_forces;
	private int oh_body;

	// attribute handles - MOP.ExternalForce
	private String[]   on_forces;
	private int[][]    oah_iInfosFor;
	private double[][] oah_dInfosFor;
	private char[][]   oah_cInfosFor;
	private double[][] oah_forces;
	private double[][] oah_attackPoints;

	// attribute handles - MOP.Body
	private double[] myBodyPos;
	private double[] myBodyVel;
	private double[] myBodyAcc;
	private double[] myBodyPosEst;
	private double[] myBodyVelEst;

	//Variabler brukt Til TwinCat I/O
	long err;
	AmsAddr addr; 
	int hdl1BuffToInt; 
	int hdl2BuffToInt; 
	JNIByteBuffer data1Buff; 
	JNIByteBuffer data2Buff; 
	Double PositionSurge;
	Double SumSurgeForce;
	Double ForceFromJava;

	PrintStream logFileH = null;
	private DatagramSocket serverSocket = null;

	public FakeSimo(double dt) {
		
		super("127.0.0.1", "HLATask1", "externalHLAfederate", dt);
	}

	@Override
	protected void publishAndSubscribeObjects() throws HLAException {
		// Informing the RTI what objects is subscribed to and what objects will be published.
		ch_body = helper.getClassHandle("MOP.Body");
		ch_force = helper.getClassHandle("MOP.ExternalForce");

		hPos = helper.getClassAttributeHandle(ch_body, "position");
		hVel = helper.getClassAttributeHandle(ch_body, "velocity");
		hAcc = helper.getClassAttributeHandle(ch_body, "acceleration");
		hPosEst = helper.getClassAttributeHandle(ch_body, "est_position");
		hVelEst = helper.getClassAttributeHandle(ch_body, "est_velocity");

		ah_iInfoFor = helper.getClassAttributeHandle(ch_force, "iInfo");
		ah_dInfoFor = helper.getClassAttributeHandle(ch_force, "dInfo");
		ah_cInfoFor = helper.getClassAttributeHandle(ch_force, "cInfo");
		ah_force    = helper.getClassAttributeHandle(ch_force, "force");
		ah_attackPoint = helper.getClassAttributeHandle(ch_force, "attackPoint");

		helper.publishClassAttributes(ch_body, new int [] {hPos, hVel, hAcc, hPosEst, hVelEst}, new int[] {i_DOUBLE, i_DOUBLE, i_DOUBLE, i_DOUBLE, i_DOUBLE});
		helper.subscribeClassAttributes(ch_force, new int[] {ah_iInfoFor, ah_dInfoFor, ah_cInfoFor, ah_attackPoint, ah_force}, new int[] {i_INT, i_DOUBLE, i_CHAR, i_DOUBLE, i_DOUBLE});
	}

	@Override
	protected void registerObjects() throws HLAException {
		// Registering objects that is published by this code

		oh_body = helper.registerObject("supportVessel", ch_body);
		helper.initializeObjectAttributes(oh_body, new int [] {hPos, hVel, hAcc, hPosEst, hVelEst}, new int[] {6, 6, 6, 6, 6});
	}

	@Override
	protected void initializeObjects() throws HLAException {
		// Initializing objects that is published by this code

		myBodyPos = helper.getDoubleAttribute(oh_body, hPos);
		myBodyVel = helper.getDoubleAttribute(oh_body, hVel);
		myBodyAcc = helper.getDoubleAttribute(oh_body, hAcc);
		myBodyPosEst = helper.getDoubleAttribute(oh_body, hPos);
		myBodyVelEst = helper.getDoubleAttribute(oh_body, hVel);

		for(int i = 0; i < 6; i++) {
			myBodyPos[i] = 0.0;
			myBodyVel[i] = 0.0;
			myBodyAcc[i] = 0.0;
			myBodyPosEst[i] = 0.0;
			myBodyVelEst[i] = 0.0;
		}
		helper.attributeUpdated(oh_body, hPos);
		helper.attributeUpdated(oh_body, hVel);
		helper.attributeUpdated(oh_body, hAcc);
		helper.attributeUpdated(oh_body, hPosEst);
		helper.attributeUpdated(oh_body, hVelEst);
		helper.sendObjectUpdates(oh_body);

		try {
			logFileH = new PrintStream("logFredrik.dat");
			//logFileH.printf("DT=%g\n",DT);
			//logFileH.printf("RAMP_STEPS=%g\n",RAMP_STEPS);
			

		} catch (FileNotFoundException e) {
			System.out.println("Error in logger initialization.");
			e.printStackTrace();
		}


		addr = new AmsAddr();
		// Handledefinisjon for vår Double (LREAL)
        JNIByteBuffer handle1Buff = new JNIByteBuffer(Integer.SIZE / Byte.SIZE);
        JNIByteBuffer symbol1Buff = new JNIByteBuffer(Convert.StringToByteArr("MAIN.LRealFromPLC",true));  //Posisjon i surge for flyter
        data1Buff = new JNIByteBuffer(Double.SIZE / Byte.SIZE);

       //  Handledefinisjon for vår Float (REAL)
        JNIByteBuffer handle2Buff = new JNIByteBuffer(Integer.SIZE / Byte.SIZE);
        JNIByteBuffer symbol2Buff = new JNIByteBuffer(Convert.StringToByteArr("MAIN.DoubleFromJava",true));  // Sum surge force for flyter
       data2Buff = new JNIByteBuffer(Double.SIZE / Byte.SIZE);
        
        // Open communication
        AdsCallDllFunction.adsPortOpen();
        AdsCallDllFunction.getLocalAddress(addr);
        addr.setPort(851);
			
	 	// Get handle by symbol name for vår Double (LREAL)
        err = AdsCallDllFunction.adsSyncReadWriteReq(addr,
                                            AdsCallDllFunction.ADSIGRP_SYM_HNDBYNAME,
                                            0x0,
                                            handle1Buff.getUsedBytesCount(),
                                            handle1Buff,
                                            symbol1Buff.getUsedBytesCount(),
                                            symbol1Buff);
		
		if(err!=0) { 
			System.out.println("Error: Get handle: 0x" 
			+ Long.toHexString(err)); 
		} else {
		System.out.println("Success: Get handle! SurgePositionFromPLC");
		}
		
		// Handle: byte[] to int
		hdl1BuffToInt = Convert.ByteArrToInt(handle1Buff.getByteArray());

		  // Get handle by symbol name for vår Float (REAL)
        err = AdsCallDllFunction.adsSyncReadWriteReq(addr,
                                            AdsCallDllFunction.ADSIGRP_SYM_HNDBYNAME,
                                            0x0,
                                            handle2Buff.getUsedBytesCount(),
                                            handle2Buff,
                                            symbol2Buff.getUsedBytesCount(),
                                            symbol2Buff);
        if(err!=0) { 
            System.out.println("Error: Get handle: 0x" 
                    + Long.toHexString(err)); 
        } else {
            System.out.println("Success: Get handle! SumSurgeForceToPLC");
        }
        // Handle: byte[] to int
        hdl2BuffToInt = Convert.ByteArrToInt(handle2Buff.getByteArray());

	}

	@Override
	protected void discoverObjects() throws HLAException {
		// Discovering objects that are subscribed to

		oh_forces = helper.getObjects(ch_force);
		int nfor = oh_forces.length;

		on_forces  = new String[nfor];
		oah_iInfosFor = new int[nfor][];
		oah_dInfosFor = new double[nfor][];
		oah_cInfosFor = new char[nfor][];
		oah_forces = new double[nfor][];
		oah_attackPoints = new double[nfor][];

		for(int ifor = 0; ifor < oh_forces.length; ifor++) {
			on_forces[ifor] = helper.getObjectName(oh_forces[ifor]);
			oah_iInfosFor[ifor] = helper.getIntegerAttribute(oh_forces[ifor], ah_iInfoFor);
			oah_dInfosFor[ifor] = helper.getDoubleAttribute(oh_forces[ifor], ah_dInfoFor);
			oah_cInfosFor[ifor] = helper.getCharacterAttribute(oh_forces[ifor], ah_cInfoFor);
			oah_forces[ifor]    = ambassador.getDoubleAttribute(oh_forces[ifor], ah_force);
			oah_attackPoints[ifor] = ambassador.getDoubleAttribute(oh_forces[ifor], ah_attackPoint);
		}
	}

	@Override
	protected void doTimeStepWork(int istep, double timeNow, double timeNext)
			throws HLAException {
			double t=timeNow; 

		System.out.printf("HLA time = %g \n",timeNow);

	 	SumSurgeForce=oah_forces[0][0]+oah_forces[1][0]; 
		//ForceFromJava= Math.round(SumSurgeForce * 1000) / 1000.0;
	 	 // Sum forces in current step
		/*for (int ifor = 0; ifor < 2; ifor++) {
		System.out.printf("Line %2d (RIFLEX) = %+8.3e\n",
			ifor+1,
			ForceFromJava
			);
		}*/
	 			
		 // Read value by handle (Double)
        err = AdsCallDllFunction.adsSyncReadReq(addr,
                                            AdsCallDllFunction.ADSIGRP_SYM_VALBYHND,
                                            hdl1BuffToInt,
                                            data1Buff.getUsedBytesCount(), // <- Antall bytes som skal leses fra PLC
                                            data1Buff);
       	 	if(err!=0)
       	 	{
       	 	    System.out.println("Error: Read by handle: 0x"
       	 	                        + Long.toHexString(err));
       	 	}
       	 	else
       	 	{
       	    // Lese en LREAL fra PLC til en Double i Java
            ByteBuffer bb = ByteBuffer.wrap(data1Buff.getByteArray()).order(ByteOrder.LITTLE_ENDIAN);
            PositionSurge = bb.getDouble();
        
            System.out.println("Success: Read Position Surge value     = " + PositionSurge); 
               	 	}
		
		logFileH.printf(" %g       %g  \n", PositionSurge,-SumSurgeForce);
				
       	ByteBuffer bb = ByteBuffer.wrap(data2Buff.getByteArray());
              
        bb.order(ByteOrder.LITTLE_ENDIAN);
        bb.putDouble(SumSurgeForce);
        
         // Write struct to PLC
        err = AdsCallDllFunction.adsSyncWriteReq(addr,
                							AdsCallDllFunction.ADSIGRP_SYM_VALBYHND,
                							hdl2BuffToInt,
               								data2Buff.getUsedBytesCount(),
                							data2Buff);
        if(err!=0) { 
            System.out.println("Error: Write request: 0x" 
                    + Long.toHexString(err)); 
        } else {
            System.out.println("Success: Write SumSurgeForceToPLC value=" + SumSurgeForce);
        }
		System.out.println("SurgePOS value=" +myBodyPos[0]);
        
		myBodyPos[0] = PositionSurge;
		myBodyPos[1] = 0;
		myBodyPos[2] = 0;
		myBodyPos[3] = 0;
		myBodyPos[4] = 0;
		myBodyPos[5] = 0;
			
		myBodyVel[0] = 0;
		myBodyVel[1] = 0;
		myBodyVel[2] = 0;
		myBodyVel[3] = 0;
		myBodyVel[4] = 0;
		myBodyVel[5] = 0;

		myBodyAcc[0] = 0;
		myBodyAcc[1] = 0;
		myBodyAcc[2] = 0;
		myBodyAcc[3] = 0;
		myBodyAcc[4] = 0;
		myBodyAcc[5] = 0;

	
		// Sending updated position and velocity data to the RTI
		helper.attributeUpdated(oh_body, hPos);
		helper.attributeUpdated(oh_body, hVel);
		helper.attributeUpdated(oh_body, hAcc);
		helper.attributeUpdated(oh_body, hPosEst);
		helper.attributeUpdated(oh_body, hVelEst);

		helper.sendObjectUpdates(oh_body, timeNext);

		}

	@Override
	protected void finishSimulation() throws HLAException {
	logFileH.close();
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
	
		FakeSimo app = new FakeSimo(DT);
		app.runSimulation(false);
	}

}
